@@ -715,3 +715,70 @@ class TestBuildSetWait:
 
         results = build_set.wait()
         assert results == [state2, state2]
+
+
+@pytest.fixture
+def results(build_attrs):
+    return tuxsuite.build.Results(
+        group=build_attrs["group"],
+        project=build_attrs["project"],
+        kbapi_url=build_attrs["kbapi_url"],
+        tuxapi_url=build_attrs["tuxapi_url"],
+        token=build_attrs["token"],
+        uid="1sewrBhxNVbsURAKBjeXX8pyjwY",
+    )
+
+
+class TestResults:
+    def test_get_build(self, results, response, get, mocker):
+        response._content = b'{"a": "b"}'
+
+        assert response.status_code == 200
+        assert results.get_build() == {"a": "b"}
+        get.assert_called_with(
+            results.tuxapi_url
+            + "/v1/groups/{}/projects/{}/builds/{}".format(
+                results.group, results.project, results.uid
+            ),
+            headers=results.headers,
+            timeout=mocker.ANY,
+            params=None,
+        )
+
+    def test_get_test(self, results, response, get, mocker):
+        url = results.tuxapi_url + "/v1/groups/{}/projects/{}/tests/{}".format(
+            results.group, results.project, results.uid
+        )
+        response._content = b'{"a": "b"}'
+
+        assert response.status_code == 200
+        assert results.get_test() == ({"a": "b"}, url)
+        get.assert_called_with(
+            url,
+            headers=results.headers,
+            timeout=mocker.ANY,
+            params=None,
+        )
+
+    def test_get_plan(self, results, response, get, mocker):
+        url = results.tuxapi_url + "/v1/groups/{}/projects/{}/plans/{}".format(
+            results.group, results.project, results.uid
+        )
+        response._content = b'{"a": "b"}'
+
+        assert response.status_code == 200
+        assert results.get_plan() == ({"a": "b"}, url.replace("plans", "tests"))
+        get.assert_called_with(
+            url,
+            headers=results.headers,
+            timeout=mocker.ANY,
+            params=None,
+        )
+
+    def test_get_all(self, results, response, get):
+        response._content = b'{"a": "b"}'
+
+        result = results.get_all()
+        assert response.status_code == 200
+        assert get.call_count == 3
+        assert isinstance(result, type(()))
