@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 from click.testing import CliRunner
+import json
 import pytest
 import tuxsuite.cli
 import tuxsuite.build
@@ -298,7 +299,8 @@ name: Simple plan
 description: A simple plan
 jobs:
 
-- builds:
+- name: tinyconfig
+  builds:
     - {toolchain: gcc-8, target_arch: i386, kconfig: tinyconfig}
     - {toolchain: gcc-9, target_arch: i386, kconfig: tinyconfig}
   test: {device: qemu-i386, tests: [ltp-smoke]}
@@ -667,10 +669,52 @@ def test_plan_cli_options(mocker, plan_config, tuxsuite_config):
     plan_obj = tuxsuite.Plan(
         "", token="token", tuxapi_url="http://api", project="prj", group="grp"
     )
-    plan_obj.builds = [1, 2]
+    plan_obj.builds = [mocker.Mock(uid="1"), mocker.Mock(uid="2")]
+    plan_obj.tests = [
+        mocker.Mock(uid="1", wait_for="1"),
+        mocker.Mock(uid="2", wait_for=None),
+    ]
     plan_obj.submit = mocker.Mock()
     plan = mocker.patch("tuxsuite.Plan")
     plan.return_value = plan_obj
+    plan_obj.results = {
+        "builds": {
+            "1": tuxsuite.build.BuildState(
+                build=None,
+                state="finished",
+                status="",
+                message="build 1 pass",
+                icon="🎉",
+                cli_color="green",
+            ),
+            "2": tuxsuite.build.BuildState(
+                build=None,
+                state="finished",
+                status="",
+                message="build 2 pass",
+                icon="🎉",
+                cli_color="green",
+            ),
+        },
+        "tests": {
+            "1": tuxsuite.build.BuildState(
+                build=None,
+                state="finished",
+                status="",
+                message="test 1 pass",
+                icon="🎉",
+                cli_color="green",
+            ),
+            "2": tuxsuite.build.BuildState(
+                build=None,
+                state="finished",
+                status="",
+                message="test 2 pass",
+                icon="🎉",
+                cli_color="green",
+            ),
+        },
+    }
 
     show_log = mocker.patch("tuxsuite.cli.show_log")
     wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
@@ -685,13 +729,38 @@ def test_plan_cli_options(mocker, plan_config, tuxsuite_config):
             str(plan_config),
         ],
     )
-    print(result.output)
+
     assert result.exit_code == 0
     plan.assert_called_once()
     assert wait_for_object.call_count == 1
     assert download.call_count == 2
     assert show_log.call_count == 2
 
+    assert (
+        """
+Summary: http://api/v1/groups/grp/projects/prj
+* 🎉 build 1 pass: None
+  -> 🎉 test 1 pass: None
+* 🎉 build 2 pass: None
+* 🎉 test 2 pass: None"""
+        in result.output
+    )
+
+
+def test_plan_empty_config(mocker, plan_config, tuxsuite_config):
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.plan,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-ref=master",
+            "--job-name=kconfig",
+            str(plan_config),
+        ],
+    )
+    assert result.exit_code == 0
+    assert "Empty plan, skipping" in result.output
+
 
 def test_test_no_args():
     runner = CliRunner()
@@ -729,6 +798,7 @@ def test_test_cli(mocker, tuxsuite_config):
         kernel="https://example.com/vmlinux",
         modules=None,
         tests=[],
+        boot_args=None,
         wait_for=None,
     )
     assert wait_for_object.call_count == 1
@@ -750,6 +820,7 @@ def test_test_cli(mocker, tuxsuite_config):
         kernel="https://example.com/vmlinux",
         modules="https://example.com/modules.tar.xz",
         tests=["ltp-smoke"],
+        boot_args=None,
         wait_for=None,
     )
     assert wait_for_object.call_count == 1
@@ -770,6 +841,7 @@ def test_test_cli(mocker, tuxsuite_config):
         kernel="",
         modules=None,
         tests=["ltp-smoke"],
+        boot_args=None,
         wait_for="mybuilduid",
     )
     assert wait_for_object.call_count == 1
@@ -803,9 +875,9 @@ def test_test_cli_errors(mocker):
     assert test.call_count == 0
 
 
-def test_build_no_wait(mocker):
+def test_build_no_wait(mocker, tmp_path):
     build = mocker.patch("tuxsuite.Build.build")
-    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    format_result = mocker.patch("tuxsuite.cli.format_result")
     runner = CliRunner()
     result = runner.invoke(
         tuxsuite.cli.build,
@@ -816,16 +888,17 @@ def test_build_no_wait(mocker):
             "--kconfig=defconfig",
             "--toolchain=gcc-9",
             "--no-wait",
+            f"--json-out={tmp_path}/build_result.json",
         ],
     )
     assert result.exit_code == 0
     assert build.call_count == 1
-    assert wait_for_object.call_count == 0
+    assert format_result.call_count == 1
 
 
-def test_build_set_no_wait(mocker, tuxsuite_config, tux_config):
+def test_build_set_no_wait(mocker, tuxsuite_config, tux_config, tmp_path):
     build = mocker.patch("tuxsuite.BuildSet.build")
-    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    format_result = mocker.patch("tuxsuite.cli.format_result")
     runner = CliRunner()
     result = runner.invoke(
         tuxsuite.cli.build_set,
@@ -835,16 +908,17 @@ def test_build_set_no_wait(mocker, tuxsuite_config, tux_config):
             f"--tux-config={tux_config}",
             "--set-name=test",
             "--no-wait",
+            f"--json-out={tmp_path}/build_result.json",
         ],
     )
     assert result.exit_code == 0
     assert build.call_count == 1
-    assert wait_for_object.call_count == 0
+    assert format_result.call_count == 9
 
 
 def test_test_cli_no_wait(mocker, tuxsuite_config):
     test = mocker.patch("tuxsuite.Test")
-    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    format_result = mocker.patch("tuxsuite.cli.format_result")
     runner = CliRunner()
     result = runner.invoke(
         tuxsuite.cli.test,
@@ -856,4 +930,342 @@ def test_test_cli_no_wait(mocker, tuxsuite_config):
     )
     assert result.exit_code == 0
     assert test.call_count == 1
-    assert wait_for_object.call_count == 0
+    assert format_result.call_count == 1
+
+
+def test_plan_cli_no_wait(mocker, tuxsuite_config, plan_config):
+    plan = mocker.patch("tuxsuite.Plan")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.plan,
+        [
+            "--git-repo=https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
+            "--git-ref=https://example.com/vmlinux",
+            f"{plan_config}",
+            "--no-wait",
+        ],
+    )
+    assert result.exit_code == 0
+    assert plan.call_count == 1
+
+
+sample_build_result = {
+    "build_name": "",
+    "download_url": "https://builds.dev.tuxbuild.com/1sMUTVjE25tQvDmjJ8SjWblpOJr/",
+    "errors_count": 0,
+    "kconfig": ["tinyconfig"],
+    "plan": "null",
+    "result": "pass",
+    "state": "finished",
+    "status_message": "build completed",
+    "target_arch": "arm",
+    "toolchain": "gcc-9",
+    "uid": "1sMUTVjE25tQvDmjJ8SjWblpOJr",
+    "warnings_count": 0,
+}
+
+
+sample_test_result = {
+    "device": "qemu-i386",
+    "plan": "1sMNB0p8a3OrpUmEBgYah0Ycer9",
+    "result": "fail",
+    "results": {"boot": "fail"},
+    "state": "finished",
+    "tests": ["boot"],
+    "uid": "1sMNBWLFrsOahyXQCNeiT4bgysP",
+}
+
+sample_all_result = {
+    "builds": {
+        "count": 3,
+        "next": "null",
+        "results": [
+            sample_build_result,
+            {**sample_build_result, **dict({"warnings_count": 1})},
+            {**sample_build_result, **dict({"warnings_count": 3})},
+            {**sample_build_result, **dict({"result": "fail", "errors_count": 1})},
+        ],
+    },
+    "plans": {
+        "count": 1,
+        "next": "null",
+        "results": [
+            {
+                "description": "A simple plan",
+                "name": "Simple plan",
+                "project": "tuxsuite/senthil",
+                "uid": "1sMNB0p8a3OrpUmEBgYah0Ycer9",
+            }
+        ],
+    },
+    "tests": {
+        "count": 1,
+        "next": "null",
+        "results": [
+            sample_test_result,
+        ],
+    },
+}
+
+
+sample_plan_status = {
+    "builds": {
+        "1sewrBhxNVbsURAKBjeXX8pyjwY": sample_build_result,
+    },
+    "tests": {
+        "1sewrJ4GHQygVKvTjUxuYRK1hOh": sample_test_result,
+    },
+}
+
+
+@pytest.fixture
+def build_result(tmp_path):
+    build_json = tmp_path / "build_result.json"
+    with build_json.open("w") as f:
+        f.write(json.dumps(sample_build_result))
+    return build_json
+
+
+@pytest.fixture
+def test_result(tmp_path):
+    test_json = tmp_path / "test_result.json"
+    with test_json.open("w") as f:
+        f.write(json.dumps(sample_test_result))
+    return test_json
+
+
+@pytest.fixture
+def build_set_result(tmp_path):
+    build_set_json = tmp_path / "build_set_result.json"
+    build_set = [
+        sample_build_result,
+        sample_build_result,
+    ]
+    with build_set_json.open("w") as f:
+        f.write(json.dumps(build_set))
+    return build_set_json
+
+
+@pytest.fixture
+def plan_result(tmp_path):
+    plan_json = tmp_path / "plan_result.json"
+    with plan_json.open("w") as f:
+        f.write(json.dumps(sample_plan_status))
+    return plan_json
+
+
+def test_results_cli(
+    mocker,
+    tmp_path,
+    tuxsuite_config,
+    build_result,
+    test_result,
+    build_set_result,
+    plan_result,
+):
+    results = mocker.patch(
+        "tuxsuite.Results.get_all", return_value=(sample_all_result, "http://api")
+    )
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 1
+
+    format_result = mocker.patch("tuxsuite.cli.format_result")
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_build", return_value=sample_build_result
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            "--build=1sMUTVjE25tQvDmjJ8SjWblpOJr",
+            f"--json-out={tmp_path}/build_result.json",
+        ],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 1
+    assert format_result.call_count == 1
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_test", return_value=(sample_test_result, "http://api")
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            "--test=1sMNBWLFrsOahyXQCNeiT4bgysP",
+        ],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 1
+    assert format_result.call_count == 1
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_plan", return_value=(sample_all_result, "http://api")
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            "--plan=1sMNB0p8a3OrpUmEBgYah0Ycer9",
+        ],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 1
+    assert format_result.call_count == 5
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_build", return_value=sample_build_result
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            f"--from-json={build_result}",
+        ],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 1
+    assert format_result.call_count == 1
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_test", return_value=(sample_test_result, "http://api")
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            f"--from-json={test_result}",
+        ],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 1
+    assert format_result.call_count == 1
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_build", return_value=sample_build_result
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            f"--from-json={build_set_result}",
+            f"--json-out={tmp_path}/build_set_result.json",
+        ],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 2
+    assert format_result.call_count == 2
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_plan", return_value=(sample_all_result, "http://api")
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            f"--from-json={plan_result}",
+        ],
+    )
+    assert result.exit_code == 0
+    assert results.call_count == 1
+    assert format_result.call_count == 5
+
+
+def test_results_cli_errors(mocker, tmp_path, tuxsuite_config, build_set_result):
+    results = mocker.patch(
+        "tuxsuite.Results.get_all",
+        return_value=(sample_all_result, "http://api"),
+        side_effect=tuxsuite.exceptions.URLNotFound,
+    )
+    format_result = mocker.patch("tuxsuite.cli.format_result")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [],
+    )
+    assert result.exit_code == 1
+    assert results.call_count == 1
+    assert format_result.call_count == 0
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_build",
+        return_value=sample_build_result,
+        side_effect=tuxsuite.exceptions.URLNotFound,
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            "--build=1sMUTVjE25tQvDmjJ8SjWblpOJr",
+        ],
+    )
+    assert result.exit_code == 1
+    assert results.call_count == 1
+    assert format_result.call_count == 0
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_test",
+        return_value=(sample_test_result, "http://api"),
+        side_effect=tuxsuite.exceptions.URLNotFound,
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            "--test=1sMNBWLFrsOahyXQCNeiT4bgysP",
+        ],
+    )
+    assert result.exit_code == 1
+    assert results.call_count == 1
+    assert format_result.call_count == 0
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_plan",
+        return_value=(sample_all_result, "http://api"),
+        side_effect=tuxsuite.exceptions.URLNotFound,
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            "--plan=1sMNB0p8a3OrpUmEBgYah0Ycer9",
+        ],
+    )
+    assert result.exit_code == 1
+    assert results.call_count == 1
+    assert format_result.call_count == 0
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results.get_build",
+        return_value=sample_build_result,
+        side_effect=tuxsuite.exceptions.URLNotFound,
+    )
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [
+            f"--from-json={build_set_result}",
+            f"--json-out={tmp_path}/build_set_result.json",
+        ],
+    )
+    assert result.exit_code == 1
+    assert results.call_count == 1
+    assert format_result.call_count == 0
+
+    mocker.resetall()
+    results = mocker.patch(
+        "tuxsuite.Results",
+        side_effect=tuxsuite.exceptions.TuxSuiteError,
+    )
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.results,
+        [],
+    )
+    assert result.exit_code == 1
+    assert results.call_count == 1
