@@ -8,57 +8,6 @@ from tuxsuite.exceptions import InvalidConfiguration
 from tuxsuite import Plan
 
 
-sample_plan_config = """
-version: 1
-name: Simple plan
-description: A simple plan
-jobs:
-
-- build: {toolchain: gcc-8, target_arch: i386, kconfig: tinyconfig}
-  test: {device: qemu-i386, tests: [ltp-smoke]}
-
-- builds:
-    - {toolchain: gcc-8, target_arch: i386, kconfig: tinyconfig}
-    - {toolchain: gcc-9, target_arch: i386, kconfig: tinyconfig}
-    - {toolchain: gcc-10, target_arch: i386, kconfig: tinyconfig}
-  test: {device: qemu-i386, tests: [ltp-smoke]}
-
-- builds:
-    - {toolchain: clang-10, target_arch: i386, kconfig: tinyconfig}
-    - {toolchain: clang-11, target_arch: i386, kconfig: tinyconfig}
-    - {toolchain: clang-nightly, target_arch: i386, kconfig: tinyconfig}
-  test: {device: qemu-i386}
-
-- build: {toolchain: clang-nightly, target_arch: i386, kconfig: tinyconfig}
-  tests:
-    - {device: qemu-i386}
-    - {device: qemu-i386, tests: [ltp-smoke]}
-
-- builds:
-    - {toolchain: gcc-8, target_arch: i386, kconfig: tinyconfig}
-    - {toolchain: gcc-9, target_arch: i386, kconfig: tinyconfig}
-    - {toolchain: gcc-10, target_arch: i386, kconfig: tinyconfig}
-  tests:
-    - {device: qemu-i386}
-    - {device: qemu-i386, tests: [ltp-smoke]}
-
-- tests:
-    - {kernel: https://storage.tuxboot.com/arm64/Image, device: qemu-arm64, tests: [ltp-smoke]}
-    - {kernel: https://storage.tuxboot.com/i386/bzImage, device: qemu-i386, tests: [ltp-smoke]}
-    - {kernel: https://storage.tuxboot.com/mips64/vmlinux, device: qemu-mips64, tests: [ltp-smoke]}
-    - {kernel: https://storage.tuxboot.com/ppc64/vmlinux, device: qemu-ppc64, tests: [ltp-smoke]}
-    - {kernel: https://storage.tuxboot.com/riscv64/Image, device: qemu-riscv64, tests: [ltp-smoke]}
-    - {kernel: https://storage.tuxboot.com/x86_64/bzImage, device: qemu-x86_64, tests: [ltp-smoke]}
-"""
-
-
-@pytest.fixture
-def plan_config(tmp_path):
-    c = tmp_path / "planv1.yml"
-    c.write_text(sample_plan_config)
-    return c
-
-
 def test_plan_config(plan_config):
     cfg = PlanConfig("", "", plan_config)
     assert cfg.name == "Simple plan"
@@ -193,6 +142,19 @@ def test_plan_config(plan_config):
     ]
 
 
+def test_plan_config_job_name(plan_config):
+    cfg = PlanConfig("hello", "world", plan_config, "gcc-simple")
+    assert len(cfg.plan) == 0
+
+    cfg = PlanConfig("hello", "world", plan_config, "simple-gcc")
+    assert len(cfg.plan) == 1
+    assert cfg.plan[0]["build"] == {
+        "kconfig": "tinyconfig",
+        "target_arch": "i386",
+        "toolchain": "gcc-8",
+    }
+
+
 def test_submit(config, plan_config, mocker):
     cfg = PlanConfig("", "", plan_config)
     plan = Plan(
@@ -496,7 +458,7 @@ def test_watch(config, mocker, plan_config):
     assert states[13].state == "finished"
 
 
-def test_config_url(get, response):
+def test_config_url(get, response, sample_plan_config):
     response._content = bytes(sample_plan_config, "utf-8")
 
     cfg = PlanConfig("hello", "world", "http://example.com/plan.yaml")
