@@ -59,3 +59,9 @@ class Test(Configurable, build.Test):
     This class represents individual tests. It should be used to trigger
     tests, and optionally wait for them to finish.
     """
+
+
+class Results(Configurable, build.Results):
+    """
+    This class represents individual results. It should be used to get results.
+    """
