@@ -45,6 +45,8 @@ def get_request(url, headers, params=None):
     if response.status_code == 200:
         return json.loads(response.text)
     else:
+        if response.status_code == 404:
+            raise tuxsuite.exceptions.URLNotFound(response.text)
         response.raise_for_status()  # Some unexpected status that's not caught
 
 
@@ -167,9 +169,9 @@ class Build(HeadersMixin):
         url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds".format(
             self.group, self.project
         )
-        json_data = post_request(url, self.headers, data)
-        self.build_data = json_data[0]["download_url"]
-        self.uid = json_data[0]["uid"]
+        self.status = post_request(url, self.headers, data)[0]
+        self.build_data = self.status["download_url"]
+        self.uid = self.status["uid"]
 
     def get_status(self):
         """Fetches the build status and updates the values inside the build object"""
@@ -375,6 +377,7 @@ class BuildSet(HeadersMixin):
         builds_by_client_token = {b.client_token: b for b in self.builds}
         for build_config in json_data:
             build = builds_by_client_token[build_config["client_token"]]
+            build.status = build_config
             build.build_data = build_config["download_url"]
             build.uid = build_config["uid"]
 
@@ -439,13 +442,17 @@ class Plan(HeadersMixin):
         self.plan = None
         self.builds = []
         self.tests = []
+        self.status = {}
         self.url = self.tuxapi_url + "/v1/groups/{}/projects/{}".format(
             self.group, self.project
         )
+        self.results = {"builds": {}, "tests": {}}
 
     def submit(self):
         builds = []
         tests = []
+        builds_status = {}
+        tests_status = {}
 
         # Create the plan
         self.plan = post_request(
@@ -474,6 +481,7 @@ class Plan(HeadersMixin):
             for (build, data) in zip(builds_to_submit, ret):
                 build.build_data = data["download_url"]
                 build.uid = data["uid"]
+                build.status = data
 
         # Create the tests
         for (build, cfg) in zip(builds, self.config.plan):
@@ -489,6 +497,7 @@ class Plan(HeadersMixin):
                             device=test["device"],
                             kernel="",
                             tests=test.get("tests", []),
+                            boot_args=test.get("boot_args"),
                             wait_for=build.uid,
                         )
                     )
@@ -506,6 +515,7 @@ class Plan(HeadersMixin):
                             kernel=test["kernel"],
                             modules=test.get("modules"),
                             tests=test.get("tests", []),
+                            boot_args=test.get("boot_args"),
                         )
                     )
 
@@ -516,8 +526,16 @@ class Plan(HeadersMixin):
         )
         for (test, data) in zip(tests, ret):
             test.uid = data["uid"]
+            test.status = data
             self.tests.append(test)
 
+        # Populate the status with builds and tests after submission
+        for build in self.builds:
+            builds_status.update({build.uid: build.status})
+        for test in self.tests:
+            tests_status.update({test.uid: test.status})
+        self.status = {"builds": builds_status, "tests": tests_status}
+
     def get_plan(self):
         builds = {}
         tests = {}
@@ -538,14 +556,17 @@ class Plan(HeadersMixin):
             start_builds = ret["builds"]["next"]
             start_tests = ret["tests"]["next"]
 
-        return {"builds": builds, "tests": tests}
+        self.status = {"builds": builds, "tests": tests}
+        return self.status
 
     def watch(self):
-        finished = {"builds": [], "tests": []}
+        n_builds = len(self.builds)
+        n_tests = len(self.tests)
+
         while True:
             plan = self.get_plan()
             for build in self.builds:
-                if build.uid not in finished["builds"]:
+                if build.uid not in self.results["builds"]:
                     build.status = plan["builds"][build.uid]
                     if not hasattr(build, "ite"):
                         build.get_status = lambda: None
@@ -554,9 +575,9 @@ class Plan(HeadersMixin):
                     if state is not None:
                         yield state
                         if state.final:
-                            finished["builds"].append(build.uid)
+                            self.results["builds"][build.uid] = state
             for test in self.tests:
-                if test.uid not in finished["tests"]:
+                if test.uid not in self.results["tests"]:
                     test.get = lambda: plan["tests"][test.uid]
                     if not hasattr(test, "ite"):
                         test.ite = test.watch(iterator=True)
@@ -564,10 +585,11 @@ class Plan(HeadersMixin):
                     if state is not None:
                         yield state
                         if state.final:
-                            finished["tests"].append(test.uid)
-            if len(finished["builds"]) == len(self.builds) and len(
-                finished["tests"]
-            ) == len(self.tests):
+                            self.results["tests"][test.uid] = state
+
+            nr_builds = len(self.results["builds"])
+            nr_tests = len(self.results["tests"])
+            if nr_builds == n_builds and nr_tests == n_tests:
                 break
             time.sleep(5)
 
@@ -590,15 +612,18 @@ class Test(HeadersMixin):
     kernel = attrib()
     modules = attrib(default=None)
     tests = attrib(default=[])
+    boot_args = attrib(default=None)
     wait_for = attrib(default=None)
     uid = attrib(default=None)
+    status = attrib(default={})
+    url = attrib(default=None)
 
     def __str__(self):
         tests = "[" + ", ".join(["boot"] + self.tests) + "]"
-        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/tests/{}".format(
+        self.url = self.tuxapi_url + "/v1/groups/{}/projects/{}/tests/{}".format(
             self.group, self.project, self.uid
         )
-        return "{} {} @ {}".format(tests, self.device, url)
+        return "{} {} @ {}".format(tests, self.device, self.url)
 
     def generate_test_request(self, plan=None):
         data = {"device": self.device, "kernel": self.kernel}
@@ -606,6 +631,8 @@ class Test(HeadersMixin):
             data["modules"] = self.modules
         if self.tests:
             data["tests"] = self.tests
+        if self.boot_args:
+            data["boot_args"] = self.boot_args
         if self.wait_for:
             data["waiting_for"] = self.wait_for
         if plan is not None:
@@ -618,14 +645,13 @@ class Test(HeadersMixin):
             self.group, self.project
         )
         data = self.generate_test_request(plan=plan)
-        data = post_request(url, self.headers, data)
-        self.uid = data["uid"]
+        self.status = post_request(url, self.headers, data)
+        self.uid = self.status["uid"]
+        self.url = url + "/{}".format(self.uid)
 
     def get(self):
-        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/tests/{}".format(
-            self.group, self.project, self.uid
-        )
-        return get_request(url, self.headers)
+        self.status = get_request(self.url, self.headers)
+        return self.status
 
     def watch(self, iterator=False):
         waiting_states = {
@@ -707,3 +733,46 @@ class Test(HeadersMixin):
         for s in self.watch():
             state = s
         return state
+
+
+@attrs(kw_only=True)
+class Results(HeadersMixin):
+    token = attrib()
+    kbapi_url = attrib()
+    tuxapi_url = attrib()
+    group = attrib()
+    project = attrib()
+    uid = attrib(default=None)
+
+    def get_build(self):
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds/{}".format(
+            self.group, self.project, self.uid
+        )
+        return get_request(url, self.headers)
+
+    def get_test(self):
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/tests/{}".format(
+            self.group, self.project, self.uid
+        )
+        result = get_request(url, self.headers)
+        return (result, url)
+
+    def get_plan(self):
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/plans/{}".format(
+            self.group, self.project, self.uid
+        )
+        result = get_request(url, self.headers)
+        return (result, url.replace("plans", "tests"))
+
+    def get_all(self):
+        all_results = {}
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}".format(
+            self.group, self.project
+        )
+        build_url = url + "/builds"
+        test_url = url + "/tests"
+        plan_url = url + "/plans"
+        all_results["builds"] = get_request(build_url, self.headers)
+        all_results["tests"] = get_request(test_url, self.headers)
+        all_results["plans"] = get_request(plan_url, self.headers)
+        return (all_results, test_url)
