@@ -11,7 +11,11 @@ import tuxsuite.exceptions
 import tuxsuite.gitutils
 
 
-from tuxsuite.utils import defaults
+from tuxsuite.utils import (
+    defaults,
+    ResultState,
+    result_states,
+)
 
 info = click.echo
 
@@ -30,24 +34,26 @@ def no_info(_):
 
 def quiet_output(quiet):
     global info
-    if quiet:
-        info = no_info
+    info = no_info if quiet else click.echo
+
+
+def print_state(state, prefix=""):
+    msg = click.style(
+        f"{prefix}{state.icon} {state.message}: ", fg=state.cli_color, bold=True
+    ) + str(state.build)
+
+    if state.status == "fail" or state.state == "error" or state.warnings:
+        warning(msg)
+    else:
+        info(msg)
 
 
 def wait_for_object(build_object):
     result = True
     for state in build_object.watch():
-        msg = click.style(
-            f"{state.icon} {state.message}: ", fg=state.cli_color, bold=True
-        ) + str(state.build)
-        if state.status == "fail" or state.state == "error":
-            warning(msg)
-            if state.final:
-                result = False
-        elif state.warnings:
-            warning(msg)
-        else:
-            info(msg)
+        print_state(state)
+        if state.status == "fail" or state.state == "error" and state.final:
+            result = False
     return result
 
 
@@ -72,6 +78,78 @@ def get_make_targets_vars(targets):
     return (target_list, make_variables)
 
 
+def format_result(result_json, tuxapi_tests_url=None):
+    state = result_states.get(result_json["state"], None)
+    result = result_json["result"]
+    result_msg = ""
+    if "build_name" in result_json:
+        result_msg = (
+            f"{result_json['target_arch']} "
+            f"({','.join(result_json['kconfig'])}) "
+            f"with {result_json['toolchain']} @ {result_json['download_url']}"
+        )
+    elif "tests" in result_json:
+        result_msg = (
+            f"[{','.join(result_json['tests'])}] "
+            f"{result_json['device']} @ {tuxapi_tests_url}"
+        )
+    if state is None:
+        errors = 0
+        warnings = 0
+
+        if result == "pass":
+            warnings = result_json.get("warnings_count", 0)
+            if warnings == 0:
+                icon = "🎉"
+                message = "Pass"
+                cli_color = "green"
+            else:
+                icon = "👾"
+                cli_color = "yellow"
+                if warnings == 1:
+                    message = "Pass (1 warning)"
+                else:
+                    message = "Pass ({} warnings)".format(warnings)
+        elif result == "fail":
+            icon = "👹"
+            cli_color = "bright_red"
+            errors = result_json.get("errors_count", 0)
+            if errors == 1:
+                message = "Fail (1 error)"
+            else:
+                message = "Fail ({} errors)".format(errors)
+            if "tests" in result_json:
+                errors = [
+                    name
+                    for name in result_json["results"]
+                    if result_json["results"][name] == "fail"
+                ]
+                message = "Fail ({})".format(", ".join(errors))
+                errors = len(errors)
+        else:
+            icon = "🔧"
+            cli_color = "bright_red"
+            message = result_json["status_message"]
+        state = ResultState(
+            state=state,
+            status=result_json["state"],
+            final=True,
+            message=message,
+            icon=icon,
+            cli_color=cli_color,
+            warnings=warnings,
+            errors=errors,
+        )
+    msg = (
+        click.style(f"{state.icon} {state.message}: ", fg=state.cli_color, bold=True)
+        + result_msg
+    )
+    if result == "fail" or result == "error":
+        warning(msg)
+    else:
+        info(msg)
+
+
 @click.group(name="tuxsuite")
 @click.version_option()  # Implement --version
 def cli():
@@ -99,7 +177,7 @@ def common_options(required):
         ),
         option(
             "--target-arch",
-            help="Target architecture [arc|arm|arm64|i386|mips|parisc|powerpc|riscv|s390|sh|sparc|x86_64]",
+            help="Target architecture [arc|arm|arm64|hexagon|i386|mips|parisc|powerpc|riscv|s390|sh|sparc|x86_64]",
         ),
         option(
             "--kernel-image",
@@ -112,7 +190,7 @@ def common_options(required):
         ),
         option(
             "--toolchain",
-            help="Toolchain [gcc-8|gcc-9|gcc-10|clang-10|clang-11|clang-12|clang-nightly|clang-android]",
+            help="Toolchain [gcc-8|gcc-9|gcc-10|gcc-11|clang-10|clang-11|clang-12|clang-nightly|clang-android]",
         ),
         option(
             "--build-name",
@@ -246,10 +324,14 @@ def build(
         build.build()
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
-    if no_wait:
-        return
 
-    build_result = wait_for_object(build)
+    build_result = True
+
+    if no_wait:
+        format_result(build.status)
+    else:
+        build_result = wait_for_object(build)
+
     if json_out:
         json_out.write(json.dumps(build.status, sort_keys=True, indent=4))
     if download:
@@ -258,6 +340,7 @@ def build(
         show_log(build, download, output_dir)
     if quiet:
         print(build.build_data)
+
     if not build_result:
         sys.exit(1)
 
@@ -298,7 +381,15 @@ def build_set(
         build_set.build()
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
+
     if no_wait:
+        build_status_list = []
+        for build in build_set.builds:
+            format_result(build.status)
+            if json_out:
+                build_status_list.append(build.status)
+        if build_status_list:
+            json_out.write(json.dumps(build_status_list, sort_keys=True, indent=4))
         return
 
     build_set_result = wait_for_object(build_set)
@@ -351,6 +442,7 @@ TESTS = ["boot", "ltp-smoke"]
     help="Comma separated list of tests",
     default="boot",
 )
+@click.option("--boot-args", help="Extra boot arguments", default=None, type=str)
 @click.option("--wait-for", help="Wait for a test uid", default=None, type=str)
 @click.option(
     "-n",
@@ -359,7 +451,12 @@ TESTS = ["boot", "ltp-smoke"]
     is_flag=True,
     help="Don't wait for tests to finish",
 )
-def test(device, kernel, modules, tests, wait_for, no_wait):
+@click.option(
+    "--json-out",
+    help="Write json test status out to a named file path",
+    type=click.File("w", encoding="utf-8"),
+)
+def test(device, kernel, modules, tests, boot_args, wait_for, no_wait, json_out):
     tests = [test for test in tests.split(",") if test]
     invalid_tests = [test for test in tests if test not in TESTS]
     if invalid_tests:
@@ -394,6 +491,7 @@ def test(device, kernel, modules, tests, wait_for, no_wait):
             kernel=kernel,
             modules=modules,
             tests=tests,
+            boot_args=boot_args,
             wait_for=wait_for,
         )
     except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
@@ -403,10 +501,16 @@ def test(device, kernel, modules, tests, wait_for, no_wait):
         test.test()
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
-    if no_wait:
-        return
 
-    test_result = wait_for_object(test)
+    test_result = True
+
+    if no_wait:
+        format_result(test.status, test.url)
+    else:
+        test_result = wait_for_object(test)
+
+    if json_out:
+        json_out.write(json.dumps(test.status, sort_keys=True, indent=4))
 
     # If the test did not pass, exit with exit code of 1
     if not test_result:
@@ -416,6 +520,7 @@ def test(device, kernel, modules, tests, wait_for, no_wait):
 @cli.command(help="Run the specified plan file", short_help="Run a plan file.")
 @click.option("--name", default=None, help="Set name")
 @click.option("--description", default=None, help="Set description")
+@click.option("--job-name", default=None, help="Job name")
 @click.option("--git-repo", default=None, help="Git repository")
 @click.option("--git-ref", default=None, help="Git reference")
 @click.option("--git-sha", default=None, help="Git commit")
@@ -445,14 +550,29 @@ def test(device, kernel, modules, tests, wait_for, no_wait):
     is_flag=True,
     help="Prints build logs to stderr in case of warnings or errors",
 )
+@click.option(
+    "-n",
+    "--no-wait",
+    default=False,
+    is_flag=True,
+    help="Don't wait for plan to finish",
+)
+@click.option(
+    "--json-out",
+    help="Write json results out to a named file path",
+    type=click.File("w", encoding="utf-8"),
+)
 @click.argument("config")
 def plan(
     name,
     description,
+    job_name,
     config,
     show_logs=None,
     download=False,
     output_dir=None,
+    no_wait=False,
+    json_out=None,
     **build_params,
 ):
     if build_params["git_head"]:
@@ -465,7 +585,10 @@ def plan(
     del build_params["git_head"]
 
     try:
-        plan_config = tuxsuite.config.PlanConfig(name, description, config)
+        plan_config = tuxsuite.config.PlanConfig(name, description, config, job_name)
+        if not plan_config.plan:
+            warning("Empty plan, skipping")
+            return
         plan = tuxsuite.Plan(plan_config, **build_params)
     except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
         error(e)
@@ -478,16 +601,33 @@ def plan(
 
     try:
         plan.submit()
-        info("Plan {}/plans/{}".format(plan.url, plan.plan))
+        info("Plan {}/plans/{}\n".format(plan.url, plan.plan))
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
 
-    result = wait_for_object(plan)
+    result = True
+
+    if no_wait:
+        for build in plan.builds:
+            format_result(build.status)
+        for test in plan.tests:
+            format_result(test.status, plan.url + "/tests/{}".format(test.uid))
+    else:
+        result = wait_for_object(plan)
+        info(f"\nSummary: {plan.url}")
+        for build in plan.builds:
+            print_state(plan.results["builds"][build.uid], prefix="* ")
+            for test in [t for t in plan.tests if t.wait_for == build.uid]:
+                print_state(plan.results["tests"][test.uid], prefix="  -> ")
+        for test in [t for t in plan.tests if t.wait_for is None]:
+            print_state(plan.results["tests"][test.uid], prefix="* ")
+
+    if json_out:
+        json_out.write(json.dumps(plan.status, sort_keys=True, indent=4))
 
     if download:
         for build in plan.builds:
             tuxsuite.download.download(build, output_dir)
-
     if show_logs:
         for build in plan.builds:
             show_log(build, download, output_dir)
@@ -496,5 +636,75 @@ def plan(
         sys.exit(1)
 
 
+@cli.command(help="Fetch results", short_help="Fetch results")
+@click.option("--build", help="UID of the build to fetch result", default="", type=str)
+@click.option("--test", help="UID of the test to fetch result", default="", type=str)
+@click.option("--plan", help="UID of the plan to fetch result", default="", type=str)
+@click.option(
+    "--from-json",
+    help="Read status input from named json file path",
+    type=click.File("r", encoding="utf-8"),
+)
+@click.option(
+    "--json-out",
+    help="Write json results out to a named file path",
+    type=click.File("w", encoding="utf-8"),
+)
+def results(build, test, plan, from_json, json_out):
+    result_json = {}
+    try:
+        results = tuxsuite.Results()
+    except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
+        error(e)
+
+    try:
+        if from_json:
+            data = json.loads(from_json.read())
+            if "builds" in data and "tests" in data:
+                plan = data["builds"][list(data["builds"].keys())[0]]["plan"]
+            elif "build_name" in data:
+                build = data["uid"]
+            elif "tests" in data:
+                test = data["uid"]
+            elif isinstance(data, list):
+                result_json = []
+                for res in data:
+                    results.uid = res["uid"]
+                    build_result = results.get_build()
+                    format_result(build_result)
+                    if json_out:
+                        result_json.append(build_result)
+        elif not any([build, test, plan]):  # get all results with no options
+            result_json, tuxapi_tests_url = results.get_all()
+            for key in result_json.keys():
+                info(f"{key.capitalize()}:")
+                for result in result_json[key].get("results", None):
+                    if key == "plans":
+                        info(f"{result['uid']}: {result['name']} {result['project']}")
+                    else:
+                        format_result(result, tuxapi_tests_url)
+                info("\n")
+        if build:
+            results.uid = build
+            result_json = results.get_build()
+            format_result(result_json)
+        if test:
+            results.uid = test
+            result_json, tuxapi_tests_url = results.get_test()
+            format_result(result_json, tuxapi_tests_url)
+        if plan:
+            results.uid = plan
+            result_json, tuxapi_tests_url = results.get_plan()
+            for result in result_json["builds"].get("results", None):
+                format_result(result)
+            for result in result_json["tests"].get("results", None):
+                format_result(result, tuxapi_tests_url=tuxapi_tests_url)
+    except tuxsuite.exceptions.URLNotFound as e:
+        raise (click.ClickException(str(e)))
+
+    if json_out:
+        json_out.write(json.dumps(result_json, sort_keys=True, indent=4))
+
+
 def main():
     cli.main(prog_name="tuxsuite")
